﻿using AVFoundation;
using CoreGraphics;
using Foundation;
using System;
using UIKit;
using Xamarin.Forms;
using Xamarin.Forms.Platform.iOS;

namespace CameraDifference.Abstractions
{
	public class CameraControl : View
	{
		AVCaptureSession _captureSession;
		AVCaptureDeviceInput _captureDeviceInput;
		UIView _liveCameraStream;
		UIButton _takePhotoButton;
		UIButton _answerButton;
		UIButton _rightButton;
		Random _rand = new Random();
		bool _flg = false;
//		Speech _speech = new Speech();

		public CameraControl()
		{
			try
			{
				SetupUserInterface();
				SetupEventHandlers();
				SetupLiveCameraStream();
				AuthorizeCameraUse();
			}
			catch (Exception ex)
			{
				System.Diagnostics.Debug.WriteLine(@"			ERROR: ", ex.Message);
			}
		}

		void SetupUserInterface ()
		{
			var bounds = View.Bounds;
			var centerButtonX = bounds.GetMidX () - 35f;
			var bottomButtonY = bounds.Bottom - 150;
			var answerY = _rand.Next((int)bounds.Height - 120);
			var rightY = (answerY < bounds.GetMidY()-71) ? (bounds.GetMidY()-50) : (bounds.GetMidY()-200);

			_liveCameraStream = new UIView()
			{
				Frame = new CGRect(0, 0, bounds.Width, bounds.Height)
			};
			_takePhotoButton = new UIButton()
			{
				Frame = new CGRect(centerButtonX, bottomButtonY, 70, 70)
			};
			_takePhotoButton.SetBackgroundImage(UIImage.FromFile("TakePhotoButton.png"), UIControlState.Normal);
			_answerButton = new UIButton()
			{
				Frame = new CGRect(_rand.Next((int)bounds.Width - 50), answerY, 50, 50)
			};
			_answerButton.SetBackgroundImage(UIImage.FromFile("AnswerButton.png"), UIControlState.Normal);
			_rightButton = new UIButton()
			{
				Frame = new CGRect(bounds.GetMidX()-150, rightY, 300, 142)
			};
			_rightButton.SetBackgroundImage(UIImage.FromFile("Right.png"), UIControlState.Normal);
			View.Add(_liveCameraStream);
			View.Add(_takePhotoButton);
		}

		void SetupEventHandlers()
		{
			_takePhotoButton.TouchUpInside += (object sender, EventArgs e) =>
			{
				var sound = new AudioToolbox.SystemSound(1108);
				sound.PlaySystemSound();
				_captureSession.StopRunning();
				_takePhotoButton.RemoveFromSuperview();
				View.Add(_answerButton);
				_flg = true;
			};
			_answerButton.TouchUpInside += (object sender, EventArgs e) =>
			{
				if ( _flg )
				{
					View.Add(_rightButton);
					_speech.Speak("Right");
				}
				_flg = false;
			};
			_rightButton.TouchUpInside += (object sender, EventArgs e) =>
			{
				NavigationController.PopViewController(true);
			};
			View.AddGestureRecognizer(new UITapGestureRecognizer(tap =>
			{
				if ( _flg )
					_speech.Speak("Wrong");
			}));
		}
																	 
		void SetupLiveCameraStream ()
		{
			_captureSession = new AVCaptureSession();
			_captureSession.SessionPreset = AVCaptureSession.PresetHigh;

			var viewLayer = _liveCameraStream.Layer;
			var videoPreviewLayer = new AVCaptureVideoPreviewLayer(_captureSession)
			{
				VideoGravity = AVLayerVideoGravity.ResizeAspectFill,
				Frame = _liveCameraStream.Bounds
			};
			_liveCameraStream.Layer.AddSublayer(videoPreviewLayer);

			var captureDevice = AVCaptureDevice.GetDefaultDevice(AVCaptureDeviceType.BuiltInWideAngleCamera, AVMediaType.Video, AVCaptureDevicePosition.Back);
			ConfigureCameraForDevice(captureDevice);
			var error = new NSError();
			_captureDeviceInput = new AVCaptureDeviceInput(captureDevice, out error);

			_captureSession.AddInput(_captureDeviceInput);
			_captureSession.StartRunning();
		}

		void ConfigureCameraForDevice (AVCaptureDevice device)
		{
			var error = new NSError ();
			if (device.IsFocusModeSupported (AVCaptureFocusMode.ContinuousAutoFocus)) {
				device.LockForConfiguration (out error);
				device.FocusMode = AVCaptureFocusMode.ContinuousAutoFocus;
				device.UnlockForConfiguration ();
			} else if (device.IsExposureModeSupported (AVCaptureExposureMode.ContinuousAutoExposure)) {
				device.LockForConfiguration (out error);
				device.ExposureMode = AVCaptureExposureMode.ContinuousAutoExposure;
				device.UnlockForConfiguration ();
			} else if (device.IsWhiteBalanceModeSupported (AVCaptureWhiteBalanceMode.ContinuousAutoWhiteBalance)) {
				device.LockForConfiguration (out error);
				device.WhiteBalanceMode = AVCaptureWhiteBalanceMode.ContinuousAutoWhiteBalance;
				device.UnlockForConfiguration ();
			}
		}

		async void AuthorizeCameraUse ()
		{
			var authorizationStatus = AVCaptureDevice.GetAuthorizationStatus (AVMediaType.Video);
			if (authorizationStatus != AVAuthorizationStatus.Authorized) {
				await AVCaptureDevice.RequestAccessForMediaTypeAsync (AVMediaType.Video);
			}
		}
	}
}

