﻿using System;
using AVFoundation;
using CoreGraphics;
using Foundation;
using UIKit;

namespace CameraDifference.iOS
{
	public class UICameraPreview : UIView
	{
		AVCaptureVideoPreviewLayer _previewLayer;
		public CameraOptions _cameraOptions;
		public AVCaptureSession _captureSession { get; set; }
		Speech_iOS _speech = new Speech_iOS();

		public UICameraPreview (CameraOptions options)
		{
			_cameraOptions = options;
			Initialize ();
			AuthorizeCameraUse();
		}

		public override void Draw (CGRect rect)
		{
			base.Draw (rect);
			_previewLayer.Frame = rect;
		}

		public override void TouchesBegan(NSSet touches, UIEvent evt)
		{
			base.TouchesBegan(touches, evt);
			if (_cameraOptions == CameraOptions.Spot)
				_speech.Speak("Wrong");
		}

		void Initialize ()
		{
			_captureSession = new AVCaptureSession ();
			_captureSession.SessionPreset = AVCaptureSession.PresetHigh;
			_previewLayer = new AVCaptureVideoPreviewLayer (_captureSession) {
				Frame = Bounds,
				VideoGravity = AVLayerVideoGravity.ResizeAspectFill
			};

			var videoDevice = AVCaptureDevice.GetDefaultDevice(AVCaptureDeviceType.BuiltInWideAngleCamera, AVMediaType.Video, AVCaptureDevicePosition.Back);
			ConfigureCameraForDevice(videoDevice);

			NSError error;
			var input = new AVCaptureDeviceInput (videoDevice, out error);
			_captureSession.AddInput (input);
			Layer.AddSublayer (_previewLayer);
			_captureSession.StartRunning ();
		}

		void ConfigureCameraForDevice(AVCaptureDevice device)
		{
			var error = new NSError();
			if (device.IsFocusModeSupported(AVCaptureFocusMode.ContinuousAutoFocus))
			{
				device.LockForConfiguration(out error);
				device.FocusMode = AVCaptureFocusMode.ContinuousAutoFocus;
				device.UnlockForConfiguration();
			}
			else if (device.IsExposureModeSupported(AVCaptureExposureMode.ContinuousAutoExposure))
			{
				device.LockForConfiguration(out error);
				device.ExposureMode = AVCaptureExposureMode.ContinuousAutoExposure;
				device.UnlockForConfiguration();
			}
			else if (device.IsWhiteBalanceModeSupported(AVCaptureWhiteBalanceMode.ContinuousAutoWhiteBalance))
			{
				device.LockForConfiguration(out error);
				device.WhiteBalanceMode = AVCaptureWhiteBalanceMode.ContinuousAutoWhiteBalance;
				device.UnlockForConfiguration();
			}
		}
	
		async void AuthorizeCameraUse()
		{
			var authorizationStatus = AVCaptureDevice.GetAuthorizationStatus(AVMediaType.Video);
			if (authorizationStatus != AVAuthorizationStatus.Authorized)
			{
				await AVCaptureDevice.RequestAccessForMediaTypeAsync(AVMediaType.Video);
			}
		}
	}
}
